@@ -1,5 +1,6 @@
 """The base Simon models"""
 
+from collections import defaultdict
 import sys
 import warnings
 
@@ -21,6 +22,8 @@ __all__ = ('Model',)
 # update before the write is considered successful.
 
 def _set_write_concern_as_safe(options, force_safe):
+    """Sets the safe parameter for write concern."""
+
     safe = options.pop('safe', None)
     w = options.pop('w', None)
 
@@ -35,6 +38,8 @@ def _set_write_concern_as_safe(options, force_safe):
 
 
 def _set_write_concern_as_w(options, force_safe):
+    """Sets w safe parameter for write concern."""
+
     safe = options.pop('safe', None)
     w = options.pop('w', None)
 
@@ -432,7 +437,7 @@ class Model(object):
         .. versionadded:: 0.1.0
         """
 
-        # There needs to be something to update
+        # There needs to be something to update.
         if field is None and not fields:
             raise ValueError('No fields have been specified.')
 
@@ -440,7 +445,7 @@ class Model(object):
 
         # Both the field/value parameters and **fields can be used for
         # the update, so build a dictionary containing all of the fields
-        # to increment and the value to increment each by
+        # to increment and the value to increment each by.
         if field is not None:
             update[field] = value
         for k, v in fields.items():
@@ -448,6 +453,194 @@ class Model(object):
 
         self._update({'$inc': update}, safe=safe)
 
+    def pop(self, fields, safe=False):
+        """Performs an atomic pop.
+
+        Values can be popped from either the end or the beginning of a
+        list. To pop a value from the end of a list, specify the name of
+        the field. The pop a  value from the beginning of a list,
+        specify the name of the field with a ``-`` in front of it.
+
+        If the document does not have an ``_id``--this will
+        most likely indicate that the document has never been saved--
+        a :class:`TypeError` will be raised.
+
+        :param fields: The names of the fields to pop from.
+        :type fields: str, list, or tuple.
+        :param safe: (optional) Whether to perform the update in safe
+                     mode.
+        :type safe: bool.
+        :raises: :class:`TypeError`
+
+        .. versionadded:: 0.5.0
+        """
+
+        if not fields:
+            raise ValueError
+
+        if not isinstance(fields, (list, tuple)):
+            fields = (fields,)
+
+        update = {}
+
+        for field in fields:
+            if field[0] == '-':
+                field = field[1:]
+                direction = -1
+            else:
+                direction = 1
+
+            update[field] = direction
+
+        self._update({'$pop': update}, safe=safe)
+
+    def pull(self, field=None, value=None, safe=False, **fields):
+        """Performs an atomic pull.
+
+        With MongoDB there are two types of pull operations: ``$pull``
+        and ``$pullAll``. As the name implies, ``$pullAll`` is intended
+        to pull all values in a list from the field, while ``$pull`` is
+        meant for single values.
+
+        This method will determine the correct operator(s) to use based
+        on the value(s) being pulled. Updates can consist of either
+        operator alone or both together.
+
+        This can be used to update a single field::
+
+            >>> obj.pull(field, value)
+
+        or to update multiple fields at a time::
+
+            >>> obj.pull(field1=value1, field2=value2)
+
+        If the document does not have an ``_id``--this will
+        most likely indicate that the document has never been saved--
+        a :class:`TypeError` will be raised.
+
+        If no fields are indicated--either through ``field`` or through
+        ``**fields``, a :class:`ValueError` will be raised.
+
+        :param field: (optional) Name of the field to pull from.
+        :type field: str.
+        :param value: (optional) Value to pull from ``field``.
+        :type value: scalar or list.
+        :param safe: (optional) Whether to perform the update in safe
+                     mode.
+        :type safe: bool.
+        :param \*\*fields: Keyword arguments specifying fields and
+                           the values to pull.
+        :type \*\*fields: \*\*kwargs.
+        :raises: :class:`TypeError`, :class:`ValueError`
+
+        .. versionadded:: 0.5.0
+        """
+
+        # There needs to be something to update.
+        if not (field and value) and not fields:
+            raise ValueError('No fields have been specified.')
+
+        # Most of the atomic methods treat update as a dict. A
+        # defaultdict of dicts is being used here because, unlike most
+        # of these, not only are two operators possible, but they can be
+        # used together. defaultdict saves the trouble of having to
+        # check for each operator in update before adding new keys to
+        # the operator's dict.
+        update = defaultdict(dict)
+
+        # Both the field/value parameters and **fields can be used for
+        # the update, so build a dictionary containing all of the fields
+        # to pull from and the value(s) to pull from each.
+        if not (field is None or value is None):
+            fields[field] = value
+
+        for k, v in fields.items():
+            if isinstance(v, (list, tuple)):
+                update['$pullAll'][k] = v
+            else:
+                update['$pull'][k] = v
+
+        self._update(update, safe=safe)
+
+    def push(self, field=None, value=None, allow_duplicates=True, safe=False,
+             **fields):
+        """Performs an atomic push.
+
+        With MongoDB there are three types of push operations:
+        ``$push``, ``$pushAll``, add ``$addToSet``. As the name implies,
+        ``$pushAll`` is intended to push all values from a list to the
+        field, while ``$push`` is meant for single values. ``$addToSet``
+        can be used with either type of value, but it will only add a
+        value to the list if it doesn't already contain the value.
+
+        This method will determine the correct operator(s) to use based
+        on the value(s) being pushed. Setting ``allow_duplicates`` to
+        ``False`` will use ``$addToSet`` instead of ``$push`` and
+        ``$pushAll``. Updates that allow duplicates can combine
+        ``$push`` and ``$pushAll`` together.
+
+        This can be used to update a single field::
+
+            >>> obj.push(field, value)
+
+        or to update multiple fields at a time::
+
+            >>> obj.push(field1=value1, field2=value2)
+
+        If the document does not have an ``_id``--this will
+        most likely indicate that the document has never been saved--
+        a :class:`TypeError` will be raised.
+
+        If no fields are indicated--either through ``field`` or through
+        ``**fields``, a :class:`ValueError` will be raised.
+
+        :param field: (optional) Name of the field to push to.
+        :type field: str.
+        :param value: (optional) Value to push to ``field``.
+        :type value: scalar or list.
+        :param allow_duplicates: (optional) Whether to allow duplicate
+                                 values to be added to the list
+        :type allow_duplicates: bool.
+        :param safe: (optional) Whether to perform the update in safe
+                     mode.
+        :type safe: bool.
+        :param \*\*fields: Keyword arguments specifying fields and
+                           the values to push.
+        :type \*\*fields: \*\*kwargs.
+        :raises: :class:`TypeError`, :class:`ValueError`
+
+        .. versionadded:: 0.5.0
+        """
+
+        # There needs to be something to update.
+        if not (field and value) and not fields:
+            raise ValueError('No fields have been specified.')
+
+        # Most of the atomic methods treat update as a dict. A
+        # defaultdict of dicts is being used here because, unlike most
+        # of these, not only are two operators possible, but they can be
+        # used together. defaultdict saves the trouble of having to
+        # check for each operator in update before adding new keys to
+        # the operator's dict.
+        update = defaultdict(dict)
+
+        # Both the field/value parameters and **fields can be used for
+        # the update, so build a dictionary containing all of the fields
+        # to push to and the value(s) to push to each.
+        if not (field is None or value is None):
+            fields[field] = value
+
+        for k, v in fields.items():
+            if isinstance(v, (list, tuple)):
+                if allow_duplicates:
+                    update['$pushAll'][k] = v
+                else:
+                    update['$addToSet'][k] = {'$each': v}
+            else:
+                update['$push' if allow_duplicates else '$addToSet'][k] = v
+
+        self._update(update, safe=safe)
+
     def raw_update(self, fields, safe=False):
         """Performs an update using a raw document.
 
@@ -469,7 +662,7 @@ class Model(object):
 
         :param fields: The document to save to the database.
         :type fields: dict.
-        :param safe: (optional) Whether to perform the save in safe
+        :param safe: (optional) Whether to perform the update in safe
                      mode.
         :type safe: bool.
         :raises: :class:`TypeError`
@@ -499,7 +692,7 @@ class Model(object):
 
         :param fields: The names of the fields to remove.
         :type fields: str, list, or tuple.
-        :param safe: (optional) Whether to perform the save in safe
+        :param safe: (optional) Whether to perform the update in safe
                      mode.
         :type safe: bool.
         :raises: :class:`TypeError`
@@ -518,6 +711,59 @@ class Model(object):
 
         self._update({'$unset': fields}, safe=safe)
 
+    def rename(self, field_from=None, field_to=None, safe=False, **fields):
+        """Performs an atomic rename.
+
+        This can be used to update a single field::
+
+            >>> obj.rename(original, new)
+
+        or to update multiple fields at a time::
+
+            >>> obj.increment(original1=new1, original2=new2)
+
+        Note that the latter does **not** set the values of the fields,
+        but rather specifies the name they should be renamed to.
+
+        If the document does not have an ``_id``--this will
+        most likely indicate that the document has never been saved--
+        a :class:`TypeError` will be raised.
+
+        If no fields are indicated--either through ``field_from`` and
+        ``field_to`` or through ``**fields``, a :class:`ValueError` will
+        be raised.
+
+        :param field_from: (optional) Name of the field to rename.
+        :type field_from: str.
+        :param field_to: (optional) New name for ``field_from``.
+        :type field_to: int.
+        :param safe: (optional) Whether to perform the update in safe
+                     mode.
+        :type safe: bool.
+        :param \*\*fields: Keyword arguments specifying fields and their
+                           new names.
+        :type \*\*fields: \*\*kwargs.
+        :raises: :class:`TypeError`, :class:`ValueError`
+
+        .. versionadded:: 0.5.0
+        """
+
+        # There needs to be something to update.
+        if not (field_from and field_to) and not fields:
+            raise ValueError('No fields have been specified.')
+
+        update = {}
+
+        # Both the field/value parameters and **fields can be used for
+        # the update, so build a dictionary containing all of the fields
+        # to rename and the names to rename each to.
+        if not (field_from is None or field_to is None):
+            update[field_from] = field_to
+        for k, v in fields.items():
+            update[k] = v
+
+        self._update({'$rename': update}, safe=safe)
+
     def save(self, safe=False):
         """Saves the object to the database.
 
@@ -598,7 +844,7 @@ class Model(object):
 
         :param fields: The names of the fields to update.
         :type fields: str, list, or tuple.
-        :param safe: (optional) Whether to perform the save in safe
+        :param safe: (optional) Whether to perform the update in safe
                      mode.
         :type safe: bool.
         :raises: :class:`AttributeError`, :class:`TypeError`
@@ -635,7 +881,7 @@ class Model(object):
         Unlike :meth:`~simon.Model.save`, ``modified`` will not be
         updated.
 
-        :param safe: (optional) Whether to perform the save in safe
+        :param safe: (optional) Whether to perform the update in safe
                      mode.
         :type safe: bool.
         :param \*\*fields: The fields to update.
@@ -812,8 +1058,13 @@ class Model(object):
 
         # Map all the field names and values
         if is_atomic(fields):
-            for k, v in fields.iteritems():
+            for k, v in fields.items():
                 fields[k] = map_field_names_and_values(v)
+                if k == '$rename':
+                    for field_from, field_to in fields[k].items():
+                        mapped = map_fields(cls, {field_to: 1},
+                                            flatten_keys=True)
+                        fields[k][field_from] = mapped.keys()[0]
         else:
             fields = map_field_names_and_values(fields)
         # When placing fields in kwargs, make a copy so changes to
@@ -832,6 +1083,10 @@ class Model(object):
                     if any(k in self._meta.required_fields for k
                             in fields['$unset']):
                         has_required_fields = False
+                elif '$rename' in fields:
+                    if any(k in self._meta.required_fields for k
+                            in fields['$rename']):
+                        has_required_fields = False
             else:
                 # For a document replacement, all of the required fields
                 # must appear in the update.
@@ -889,6 +1144,17 @@ class Model(object):
                     else:
                         self._document.pop(k, None)
 
+            rename = fields.get('$rename')
+            if rename:
+                # The old fields need to be removed. The new fields will
+                # be added through the find_one() below.
+                for k in rename.keys():
+                    if '.' in k:
+                        self._document = remove_nested_key(self._document, k)
+                    else:
+                        self._document.pop(k, None)
+                fields['$rename'] = dict((v, 1) for v in rename.values())
+
             # If the only operation was an $unset, we're done.
             if not fields:
                 return
