@@ -9,17 +9,12 @@ from contextlib import nested
 from datetime import datetime
 import warnings
 
-from bson import ObjectId
 import mock
 
 from simon import Model, connection
 from simon.query import Q
 
-from .utils import ModelFactory
-
-AN_OBJECT_ID_STR = '50d4dce70ea5fae6fb84e44b'
-AN_OBJECT_ID = ObjectId(AN_OBJECT_ID_STR)
-
+from .utils import AN_OBJECT_ID, ModelFactory
 
 DefaultModel = ModelFactory('DefaultModel')
 MappedModel = ModelFactory('ModelFactory', field_map={'fake': 'real'})
@@ -34,13 +29,6 @@ class TestBase(unittest.TestCase):
         with mock.patch('simon.connection.MongoClient'):
             cls.connection = connection.connect('localhost', name='test-simon')
 
-    def tearDown(self):
-        # Even though warnings are only checked in a few tests, it's a
-        # good idea to reset them here just in case there's an exception
-        # in one of the tests
-        warnings.resetwarnings()
-        warnings.simplefilter('ignore')
-
     @classmethod
     def tearDownClass(cls):
         # Reset the cached connections and databases so the ones added
@@ -192,28 +180,34 @@ class TestBase(unittest.TestCase):
     def test_find_deprecationwarning(self):
         """Test that `find()` triggers `DeprecationWarning`."""
 
-        warnings.simplefilter('error', DeprecationWarning)
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
 
-        with mock.patch.object(DefaultModel, '_find'):
-            with self.assertRaises(DeprecationWarning) as e:
+            with mock.patch.object(DefaultModel, '_find'):
                 DefaultModel.find(Q(a=1), Q(b=2))
 
-        expected = 'qs has been deprecated. Please use q instead.'
-        actual = str(e.exception)
-        self.assertEqual(actual, expected)
+            self.assertEqual(len(w), 1)
+            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
+
+            expected = 'qs has been deprecated. Please use q instead.'
+            actual = str(w[-1].message)
+            self.assertEqual(actual, expected)
 
     def test_get_deprecationwarning(self):
         """Test that `get()` triggers `DeprecationWarning`."""
 
-        warnings.simplefilter('error', DeprecationWarning)
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
 
-        with mock.patch.object(DefaultModel, '_find'):
-            with self.assertRaises(DeprecationWarning) as e:
+            with mock.patch.object(DefaultModel, '_find'):
                 DefaultModel.get(Q(a=1), Q(b=2))
 
-        expected = 'qs has been deprecated. Please use q instead.'
-        actual = str(e.exception)
-        self.assertEqual(actual, expected)
+            self.assertEqual(len(w), 1)
+            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
+
+            expected = 'qs has been deprecated. Please use q instead.'
+            actual = str(w[-1].message)
+            self.assertEqual(actual, expected)
 
     def test_get_or_create_create(self):
         """Test the `get_or_create()` method for creating documents."""
@@ -347,6 +341,201 @@ class TestBase(unittest.TestCase):
         self.assertNotEqual(m2 == m1, m2 != m1)
         self.assertEqual(m1 != m2, m2 != m1)
 
+    def test_pop(self):
+        """Test the `pop()` method."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with mock.patch.object(DefaultModel, '_update') as _update:
+            m.pop('a')
+
+            _update.assert_called_with({'$pop': {'a': 1}}, safe=False)
+
+            m.pop(['a'])
+
+            _update.assert_called_with({'$pop': {'a': 1}}, safe=False)
+
+            m.pop('-a')
+
+            _update.assert_called_with({'$pop': {'a': -1}}, safe=False)
+
+    def test_pop_multiple(self):
+        """Test the `pop()` method with multiple fields."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with mock.patch.object(DefaultModel, '_update') as _update:
+            m.pop(('a', 'b'))
+
+            _update.assert_called_with({'$pop': {'a': 1, 'b': 1}}, safe=False)
+
+            m.pop(('a', '-b'))
+
+            _update.assert_called_with({'$pop': {'a': 1, 'b': -1}}, safe=False)
+
+            m.pop(('-a', '-b'))
+
+            _update.assert_called_with({'$pop': {'a': -1, 'b': -1}},
+                                       safe=False)
+
+    def test_pull(self):
+        """Test the `pull()` method."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with mock.patch.object(DefaultModel, '_update') as _update:
+            m.pull('a', 1)
+
+            _update.assert_called_with({'$pull': {'a': 1}}, safe=False)
+
+            m.pull('a', [1, 2])
+
+            _update.assert_called_with({'$pullAll': {'a': [1, 2]}}, safe=False)
+
+            m.pull(a=2)
+
+            _update.assert_called_with({'$pull': {'a': 2}}, safe=False)
+
+            m.pull(a=[2, 3])
+
+            _update.assert_called_with({'$pullAll': {'a': [2, 3]}}, safe=False)
+
+    def test_pull_multiple(self):
+        """Test the `pull()` method with multiple fields."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with mock.patch.object(DefaultModel, '_update') as _update:
+            m.pull(a=1, b=2)
+
+            _update.assert_called_with({'$pull': {'a': 1, 'b': 2}}, safe=False)
+
+            m.pull(a=1, b=[2, 3])
+
+            _update.assert_called_with({'$pull': {'a': 1},
+                                        '$pullAll': {'b': [2, 3]}},
+                                       safe=False)
+
+            m.pull(a=[1, 2], b=[3, 4])
+
+            _update.assert_called_with({'$pullAll': {'a': [1, 2],
+                                                     'b': [3, 4]}},
+                                       safe=False)
+
+    def test_pull_valueerror(self):
+        """Test that `pull()` raises `ValueError`."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with self.assertRaises(ValueError):
+            m.pull()
+
+        with self.assertRaises(ValueError):
+            m.pull('a')
+
+    def test_push(self):
+        """Test the `push()` method."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with mock.patch.object(DefaultModel, '_update') as _update:
+            m.push('a', 1)
+
+            _update.assert_called_with({'$push': {'a': 1}}, safe=False)
+
+            m.push('a', [1, 2])
+
+            _update.assert_called_with({'$pushAll': {'a': [1, 2]}}, safe=False)
+
+            m.push(a=2)
+
+            _update.assert_called_with({'$push': {'a': 2}}, safe=False)
+
+            m.push(a=[2, 3])
+
+            _update.assert_called_with({'$pushAll': {'a': [2, 3]}}, safe=False)
+
+    def tet_push_addtoset(self):
+        """Test the `push()` method with `$addToSet`."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with mock.patch.object(DefaultModel, '_update') as _update:
+            m.push('a', 1, allow_duplicates=False)
+
+            _update.assert_called_with({'$addToSet': {'a': 1}}, safe=False)
+
+            m.push('a', [1, 2], allow_duplicates=False)
+
+            _update.assert_called_with({'$addToSet': {'a': {'$each': [1, 2]}}},
+                                       safe=False)
+
+            m.push(a=2, allow_duplicates=False)
+
+            _update.assert_called_with({'$addToSet': {'a': 2}}, safe=False)
+
+            m.push(a=[2, 3], allow_duplicates=False)
+
+            _update.assert_called_with({'$addToSet': {'a': {'$each': [2, 3]}}},
+                                       safe=False)
+
+    def test_push_multiple(self):
+        """Test the `push()` method with multiple fields."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with mock.patch.object(DefaultModel, '_update') as _update:
+            m.push(a=1, b=2)
+
+            _update.assert_called_with({'$push': {'a': 1, 'b': 2}}, safe=False)
+
+            m.push(a=1, b=[2, 3])
+
+            _update.assert_called_with({'$push': {'a': 1},
+                                        '$pushAll': {'b': [2, 3]}},
+                                       safe=False)
+
+            m.push(a=[1, 2], b=[3, 4])
+
+            _update.assert_called_with({'$pushAll': {'a': [1, 2],
+                                                     'b': [3, 4]}},
+                                       safe=False)
+
+    def test_push_multiple_addto_set(self):
+        ("Test the `push()` method with multiple fields with "
+         "`$addToSet`.")
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with mock.patch.object(DefaultModel, '_update') as _update:
+            m.push(a=1, b=2, allow_duplicates=False)
+
+            _update.assert_called_with({'$addToSet': {'a': 1, 'b': 2}},
+                                       safe=False)
+
+            m.push(a=1, b=[2, 3], allow_duplicates=False)
+
+            _update.assert_called_with({'$addToSet': {'a': 1,
+                                                      'b': {'$each': [2, 3]}}},
+                                       safe=False)
+
+            m.push(a=[1, 2], b=[3, 4], allow_duplicates=False)
+
+            _update.assert_called_with({'$addToSet': {'a': {'$each': [1, 2]},
+                                                      'b': {'$each': [3, 4]}}},
+                                       safe=False)
+
+    def test_push_valueerror(self):
+        """Test that `push()` raises `ValueError`."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with self.assertRaises(ValueError):
+            m.push()
+
+        with self.assertRaises(ValueError):
+            m.push('a')
+
     def test_raw_update(self):
         """Test the `raw_update()` method."""
 
@@ -378,6 +567,41 @@ class TestBase(unittest.TestCase):
             _update.assert_called_with({'$unset': {'a': 1, 'b': 1}},
                                        safe=False)
 
+    def test_rename(self):
+        """Test the `rename()` method."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with mock.patch.object(DefaultModel, '_update') as _update:
+            m.rename('a', 'b')
+
+            _update.assert_called_with({'$rename': {'a': 'b'}}, safe=False)
+
+    def test_rename_multiple(self):
+        """Test the `rename()` method with multiple fields."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID, a=1, b=2)
+
+        with mock.patch.object(DefaultModel, '_update') as _update:
+            m.rename(a='b', c='d')
+
+            _update.assert_called_with({'$rename': {'a': 'b', 'c': 'd'}},
+                                       safe=False)
+
+    def test_rename_valueerror(self):
+        """Test that `rename()` raises `ValueError`."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with self.assertRaises(ValueError):
+            m.rename()
+
+        with self.assertRaises(ValueError):
+            m.rename('a')
+
+        with self.assertRaises(ValueError):
+            m.rename(field_to='a')
+
     def test_repr(self):
         """Test the `__repr__()` method."""
 
@@ -569,7 +793,6 @@ class TestBase(unittest.TestCase):
         self.assertEqual(actual, expected)
 
         m2 = StringModel()
-        print m2.__str__, str(m2)
 
         expected = u'this is the __unicode__'
         actual = u'{0}'.format(m2)
