@@ -7,16 +7,12 @@ except ImportError:
 
 from contextlib import nested
 
-from bson import ObjectId
 import mock
 import pymongo
 
 from simon import connection, query
 
-from .utils import ModelFactory
-
-AN_OBJECT_ID_STR = '50d4dce70ea5fae6fb84e44b'
-AN_OBJECT_ID = ObjectId(AN_OBJECT_ID_STR)
+from .utils import AN_OBJECT_ID, AN_OBJECT_ID_STR, ModelFactory
 
 # Set the write concern argument
 if pymongo.version_tuple[:2] >= (2, 4):
@@ -349,7 +345,7 @@ class TestDatabase(unittest.TestCase):
         with nested(mock.patch.object(MappedModel._meta.db, 'update'),
                     mock.patch.object(MappedModel._meta.db, 'find_one'),
                     ) as (update, find_one):
-            find_one.return_value = {'real': 1}
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'real': 1}
 
             m._update({'$set': {'fake': 1}})
 
@@ -385,6 +381,151 @@ class TestDatabase(unittest.TestCase):
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
                                       document={'a.b': 1}, **wc_on)
 
+    def test__update_pop(self):
+        """Test the `_update()` method with a pop."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID, a=[1, 2])
+
+        with nested(mock.patch.object(DefaultModel._meta.db, 'update'),
+                    mock.patch.object(DefaultModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': [1]}
+
+            m._update({'$pop': {'a': 1}})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'$pop': {'a': 1}}, **wc_on)
+
+            self.assertEqual(len(m._document['a']), 1)
+            self.assertNotIn(2, m._document['a'])
+
+    def test__update_pull(self):
+        """Test the `_update()` method with a pull."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID, a=[1, 2])
+
+        with nested(mock.patch.object(DefaultModel._meta.db, 'update'),
+                    mock.patch.object(DefaultModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': [2]}
+
+            m._update({'$pull': {'a': 1}})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'$pull': {'a': 1}}, **wc_on)
+
+            self.assertNotIn(1, m._document['a'])
+            self.assertIn(2, m._document['a'])
+
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': []}
+
+            m._update({'$pullAll': {'a': [2]}})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'$pullAll': {'a': [2]}},
+                                      **wc_on)
+
+            self.assertNotIn(2, m._document['a'])
+
+    def test__update_push(self):
+        """Test the `_update()` method with a push."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID)
+
+        with nested(mock.patch.object(DefaultModel._meta.db, 'update'),
+                    mock.patch.object(DefaultModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': [1]}
+
+            m._update({'$push': {'a': 1}})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'$push': {'a': 1}}, **wc_on)
+
+            self.assertIn(1, m._document['a'])
+
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': [1, 2]}
+
+            m._update({'$pushAll': {'a': [2]}})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'$pushAll': {'a': [2]}},
+                                      **wc_on)
+
+            self.assertIn(2, m._document['a'])
+
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': [1, 2, 3]}
+
+            m._update({'$addToSet': {'a': 3}})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'$addToSet': {'a': 3}},
+                                      **wc_on)
+
+            self.assertIn(3, m._document['a'])
+
+    def test__update_rename(self):
+        """Test the `_update()` method with a rename."""
+
+        m = DefaultModel(_id=AN_OBJECT_ID, a=1)
+
+        with nested(mock.patch.object(DefaultModel._meta.db, 'update'),
+                    mock.patch.object(DefaultModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'b': 1}
+
+            m._update({'$rename': {'a': 'b'}})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'$rename': {'a': 'b'}},
+                                      **wc_on)
+
+            self.assertNotIn('a', m._document)
+            self.assertIn('b', m._document)
+            self.assertEqual(m._document['b'], 1)
+
+    def test__update_rename_field_map(self):
+        ("Test the `_update()` method with a rename with a name in "
+         "`field_map`.")
+
+        m = MappedModel(_id=AN_OBJECT_ID, b=1)
+
+        with nested(mock.patch.object(MappedModel._meta.db, 'update'),
+                    mock.patch.object(MappedModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'real': 1}
+
+            m._update({'$rename': {'a': 'fake'}})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'$rename': {'a': 'real'}},
+                                      **wc_on)
+
+            self.assertNotIn('a', m._document)
+            self.assertIn('real', m._document)
+            self.assertEqual(m._document['real'], 1)
+
+    def test__update_rename_nested(self):
+        ("Test the `_update()` method with a rename with an embedded "
+         "document.")
+
+        m = DefaultModel(_id=AN_OBJECT_ID, a={'b': 1})
+
+        with nested(mock.patch.object(DefaultModel._meta.db, 'update'),
+                    mock.patch.object(DefaultModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': {'c': 1}}
+
+            m._update({'$rename': {'a.b': 'a.c'}})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'$rename': {'a.b': 'a.c'}},
+                                      **wc_on)
+
+            self.assertNotIn('b', m._document['a'])
+            self.assertIn('c', m._document['a'])
+            self.assertEqual(m._document['a']['c'], 1)
+
     def test__update_required_field(self):
         """Test the `_update()` method with a required field."""
 
@@ -405,7 +546,7 @@ class TestDatabase(unittest.TestCase):
         with nested(mock.patch.object(RequiredModel._meta.db, 'update'),
                     mock.patch.object(RequiredModel._meta.db, 'find_one'),
                     ) as (update, find_one):
-            find_one.return_value = {'a': 1, 'b': 2}
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': 1, 'b': 2}
 
             m._update({'$set': {'a': 1, 'b': 2}})
 
@@ -427,13 +568,30 @@ class TestDatabase(unittest.TestCase):
                     mock.patch.object(RequiredEmbeddedModel._meta.db,
                                       'find_one'),
                     ) as (update, find_one):
-            find_one.return_value = {'a.b': 1}
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'a': {'b': 1}}
 
             m._update({'a.b': 1})
 
             update.assert_called_with(spec={'_id': AN_OBJECT_ID},
                                       document={'a.b': 1}, **wc_on)
 
+    def test__update_required_rename(self):
+        ("Test the `_update()` method with a required field with a "
+         "rename.")
+
+        m = RequiredModel(_id=AN_OBJECT_ID, a=1, b=2, c=3)
+
+        with nested(mock.patch.object(RequiredModel._meta.db, 'update'),
+                    mock.patch.object(RequiredModel._meta.db, 'find_one'),
+                    ) as (update, find_one):
+            find_one.return_value = {'_id': AN_OBJECT_ID, 'd': 3}
+
+            m._update({'$rename': {'c': 'd'}})
+
+            update.assert_called_with(spec={'_id': AN_OBJECT_ID},
+                                      document={'$rename': {'c': 'd'}},
+                                      **wc_on)
+
     def test__update_required_field_unset(self):
         ("Test the `_update()` method with a required field with "
          "`$unset`.")
@@ -480,6 +638,14 @@ class TestDatabase(unittest.TestCase):
         actual = str(e.exception)
         self.assertEqual(actual, expected)
 
+        with self.assertRaises(TypeError) as e:
+            m3._update({'$rename': {'a': 'd'}})
+
+        expected = ("The 'RequiredModel' object cannot be updated because it "
+                    "must contain all of the required fields: a, b.")
+        actual = str(e.exception)
+        self.assertEqual(actual, expected)
+
     def test__update_use_internal(self):
         """Test the `_update()` method with `use_internal`."""
 
